(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory(global.systyle = {}));
}(this, function (exports) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    function __rest(s, e) {
        var t = {};
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
            t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
                t[p[i]] = s[p[i]];
        return t;
    }

    function isMergeable(value) {
        return value !== null && typeof value === 'object' && value.constructor === Object; // prettier-ignore
    }
    function flatten(arr, res) {
        if (res === void 0) { res = []; }
        var i, cur;
        var len = arr.length;
        for (i = 0; i < len; i++) {
            cur = arr[i];
            Array.isArray(cur) ? flatten(cur, res) : res.push(cur);
        }
        return res;
    }
    function merge(target, source) {
        if (!isMergeable(target) || !isMergeable(source))
            return source;
        var merged = __assign({}, target);
        for (var key in source) {
            merged[key] = merge(target[key], source[key]);
        }
        return merged;
    }
    function apply(acc, fn) {
        return fn(acc) || acc;
    }
    function compose(fns) {
        return function (input) { return fns.reduceRight(apply, input); };
    }
    function moulinettify(declaration) {
        return isMergeable(declaration)
            ? function (props) { return merge(declaration, props); }
            : declaration;
    }

    function createElement(_a) {
        var _b = _a.as, type = _b === void 0 ? 'div' : _b, props = __rest(_a, ["as"]);
        return __assign({}, props, { as: type });
    }

    var rxRules = /^(\&|\@|\:|\#|\.)/;
    function isCSS(_, key) {
        if (rxRules.test(key))
            return true;
        return typeof document === 'undefined'
            ? require('known-css-properties').all.includes(require('kebab-case')(key))
            : key in document.body.style;
    }
    function isVoid(value) {
        return typeof value === 'undefined' || value === null;
    }
    function isObject(value) {
        return (value !== null &&
            typeof value === 'object' &&
            value.constructor === Object &&
            !value.hasOwnProperty('$$typeof'));
    }
    function isEmpty(value) {
        return isVoid(value) || Object.keys(value).length === 0;
    }
    function partition(by) {
        return function (props) {
            var match = {};
            var rest = {};
            Object.keys(props).forEach(function (key) {
                var value = props[key];
                if (by(value, key, props)) {
                    match[key] = value;
                }
                else {
                    rest[key] = value;
                }
            });
            return [match, rest];
        };
    }
    function compact(props) {
        return partition(isVoid)(props)[1];
    }
    function replace(by) {
        var _replace = function (props, context) {
            if (context === void 0) { context = props; }
            var result = {};
            for (var key in props) {
                var parsed = by(props[key], key, context);
                var value = typeof parsed === 'undefined' ? props[key] : parsed;
                result[key] = isObject(value) ? _replace(value, context) : value;
            }
            return result;
        };
        return _replace;
    }
    function rename(aliases) {
        var _rename = function (props) {
            var renamed = {};
            Object.keys(props).forEach(function (key) {
                var value = props[key];
                var alias = key in aliases ? aliases[key] : key;
                var renamedValue = isObject(value) ? _rename(value) : value;
                if (Array.isArray(alias)) {
                    alias.forEach(function (aliasKey) { return (renamed[aliasKey] = renamedValue); });
                }
                else {
                    renamed[alias] = renamedValue;
                }
            });
            return renamed;
        };
        return _rename;
    }

    var partitionCSS = partition(isCSS);
    function extractCSS(_a) {
        var _b = _a.css, nextCSS = _b === void 0 ? [] : _b, input = __rest(_a, ["css"]);
        var _c = partitionCSS(input), cssProps = _c[0], props = _c[1];
        var definedCSS = compact(cssProps);
        var nextCSSList = Array.isArray(nextCSS) ? nextCSS : [nextCSS];
        var css = isEmpty(definedCSS) ? nextCSSList : [definedCSS].concat(nextCSSList);
        return css.length > 0 ? __assign({}, props, { css: css }) : props;
    }

    function theme(props, category, key) {
        if (!('theme' in props) || !(category in props.theme))
            return key;
        var values = props.theme[category];
        if (typeof key !== 'undefined') {
            return key in values ? values[key] : key;
        }
        else {
            return values;
        }
    }
    var rxSpacing = /margin|padding/;
    var rxColor = /color|Color/;
    var replaceCSS = function (parser) {
        return replace(function (value, key, props) {
            return isCSS(value, key) ? parser(value, key, props) : undefined;
        });
    };
    var spacing = replaceCSS(function (value, key, props) {
        if (typeof value === 'number' && rxSpacing.test(key)) {
            return value * (theme(props, 'spacing') || 1);
        }
    });
    var fonts = replaceCSS(function (value, key, props) {
        if (key === 'fontFamily') {
            return theme(props, 'fonts', value);
        }
    });
    var sizes = replaceCSS(function (value, key, props) {
        if (key === 'fontSize') {
            return theme(props, 'sizes', value);
        }
    });
    var colors = replaceCSS(function (value, key, props) {
        if (key === 'background' || rxColor.test(key)) {
            return theme(props, 'colors', value);
        }
    });
    var applyTheme = compose([spacing, fonts, sizes, colors]);

    var aliases = rename({
        bg: 'backgroundColor',
        m: 'margin',
        mx: ['marginLeft', 'marginRight'],
        my: ['marginTop', 'marginBottom'],
        mt: 'marginTop',
        mr: 'marginRight',
        mb: 'marginBottom',
        ml: 'marginLeft',
        p: 'padding',
        px: ['paddingLeft', 'paddingRight'],
        py: ['paddingTop', 'paddingBottom'],
        pt: 'paddingTop',
        pr: 'paddingRight',
        pb: 'paddingBottom',
        pl: 'paddingLeft',
        b: 'border',
        bx: ['borderLeft', 'borderRight'],
        by: ['borderTop', 'borderBottom'],
        bt: 'borderTop',
        br: 'borderRight',
        bb: 'borderBottom',
        bl: 'borderLeft',
        font: 'fontFamily',
        size: 'fontSize'
    });

    function isNumber(key) {
        return typeof key === 'number' || !isNaN(Number(key));
    }
    function isResponsive(breakpoints) {
        return function (value, key) {
            return isObject(value) && isObject(breakpoints) && key in breakpoints;
        };
    }
    function mediaQuery(key, breakpoints) {
        if (breakpoints === void 0) { breakpoints = {}; }
        var value = key in breakpoints ? breakpoints[key] : key;
        if (typeof value === 'string' && value.startsWith('@media')) {
            return value;
        }
        var breakpoint = isNumber(value) ? value + 'px' : value;
        return "@media screen and (min-width: " + breakpoint + ")";
    }
    function responsive(input) {
        var breakpoints = theme(input, 'breakpoints');
        var _a = partition(isResponsive(breakpoints))(input), responsive = _a[0], props = _a[1];
        var mediaQueries = Object.keys(responsive).map(function (key) {
            var _a;
            return (_a = {},
                _a[mediaQuery(key, breakpoints)] = responsive[key],
                _a);
        });
        return mediaQueries.reduce(merge, props);
    }

    var systyle = compose([
        createElement,
        extractCSS,
        applyTheme,
        responsive,
        aliases
    ]);

    function uid() {
        return Math.random()
            .toString(36)
            .slice(2);
    }
    function addClassName(extraClassName) {
        return function (props) { return (__assign({}, props, { className: [props.className, extraClassName].filter(Boolean).join(' ') })); };
    }

    function createStyled(builder, moulinettes) {
        if (moulinettes === void 0) { moulinettes = []; }
        var className = "s" + uid();
        var moulinette = compose([
            addClassName(className),
            systyle
        ].concat(moulinettes));
        var Styled = builder(moulinette);
        Styled.with = function () {
            var extras = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                extras[_i] = arguments[_i];
            }
            return createStyled(builder, moulinettes.concat(flatten(extras).map(moulinettify)));
        }; // prettier-ignore
        Styled.as = function (type) { return Styled.with({ as: type }); };
        Styled.className = className;
        Styled.toString = function () { return "." + className; };
        return Styled;
    }

    exports.compose = compose;
    exports.createStyled = createStyled;
    exports.default = createStyled;
    exports.merge = merge;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
